-- [CS2D] Player Trails (Lua)
-- By BCG2000
-- Description: Real player trails, made possible by the new sprite capabilities of CS2D images.
-- Version: 1.5 (September 30th, 2016)
-- Copyright  2016, BCG2000

print("[B2PT] Now loading Player Trails by BCG2000!")

-- [VARIABLES]

B2PT = {Enabled = true, Trails = {}, WImages = {}} -- Declare and initialize the main variable.
B2PT_MaxTrails = 2 -- Max trails left by each player, leave as 2 for optimum performance. Increasing this number can easily cause massive lag spikes.

for i = 1, 32 do -- I optimized this script the best I could, but players with slow connection speed may still lag regardless,
	B2PT.Trails[i] = {PX = 0, PY = 0, CTrail = 1, Trail = {}} -- because this script is expensive bandwidth-wise.
	for j = 1, B2PT_MaxTrails do -- Player image ID, current weapon, weapon image ID and current frame.
		B2PT.Trails[i].Trail[j] = {PImage = nil, CWeapon = 0, WImage = nil}
	end
end

B2PT.WImages[1] = "usp.bmp" -- Create an array with all weapon and item image names, for easier usage.
B2PT.WImages[2] = "glock.bmp" -- Only active weapons and items (that are held in the player's hands) are put here.
B2PT.WImages[3] = "deagle.bmp" -- Passive items like defuse kit and armor are not included,
B2PT.WImages[4] = "p228.bmp" -- they could be included but it would make the script more potentially laggy than it already is.
B2PT.WImages[5] = "elite.bmp"
B2PT.WImages[6] = "fiveseven.bmp"
B2PT.WImages[10] = "m3.bmp"
B2PT.WImages[11] = "xm1014.bmp"
B2PT.WImages[20] = "mp5.bmp"
B2PT.WImages[21] = "tmp.bmp"
B2PT.WImages[22] = "p90.bmp"
B2PT.WImages[23] = "mac10.bmp"
B2PT.WImages[24] = "ump45.bmp"
B2PT.WImages[30] = "ak47.bmp"
B2PT.WImages[31] = "sg552.bmp"
B2PT.WImages[32] = "m4a1.bmp"
B2PT.WImages[33] = "aug.bmp"
B2PT.WImages[34] = "scout.bmp"
B2PT.WImages[35] = "awp.bmp"
B2PT.WImages[36] = "g3sg1.bmp"
B2PT.WImages[37] = "sg550.bmp"
B2PT.WImages[38] = "galil.bmp"
B2PT.WImages[39] = "famas.bmp"
B2PT.WImages[40] = "m249.bmp"
B2PT.WImages[41] = "tacticalshield.bmp"
B2PT.WImages[45] = "laser.bmp"
B2PT.WImages[46] = "flamethrower.bmp"
B2PT.WImages[47] = "rpglauncher.bmp"
B2PT.WImages[48] = "rocketlauncher.bmp"
B2PT.WImages[49] = "grenadelauncher.bmp"
B2PT.WImages[50] = "knife.bmp"
B2PT.WImages[51] = "he.bmp"
B2PT.WImages[52] = "flashbang.bmp"
B2PT.WImages[53] = "smokegrenade.bmp"
B2PT.WImages[54] = "flare.bmp"
B2PT.WImages[55] = "bomb.bmp"
B2PT.WImages[69] = "machete.bmp"
B2PT.WImages[72] = "gasgrenade.bmp"
B2PT.WImages[73] = "molotovcocktail.bmp"
B2PT.WImages[74] = "wrench.bmp"
B2PT.WImages[75] = "snowball.bmp"
B2PT.WImages[76] = "airstrike.bmp"
B2PT.WImages[77] = "mine.bmp"
B2PT.WImages[78] = "claw.bmp"
B2PT.WImages[86] = "gutbomb.bmp"
B2PT.WImages[87] = "lasermine.bmp"
B2PT.WImages[88] = "portalgun.bmp"
B2PT.WImages[90] = "m134.bmp"
B2PT.WImages[91] = "fnf2000.bmp"

-- [OPTIMIZATION]

-- Localize math functions for faster math operations
local rad = math.rad
local cos = math.cos
local sin = math.sin

-- [FUNCTIONS]

function PlayerIsUsingPrimaryWeapon(id) -- Check if the specified player is using any primary weapon.
	local PWeapon = player(id, "weapontype")
	if ((PWeapon > 9 and PWeapon < 12) or (PWeapon > 19 and PWeapon < 25) or (PWeapon > 29 and PWeapon < 41)) then
		PWeapon = nil
		return true
	end
	PWeapon = nil
	return false
end

function PlayerIsUsingSecondaryWeapon(id) -- Check if the specified player is using any secondary weapon.
	local PWeapon = player(id, "weapontype")
	if ((PWeapon > 0 and PWeapon < 7)) then
		PWeapon = nil
		return true
	end
	PWeapon = nil
	return false
end

function PlayerIsUsingThrowable(id) -- Check if the specified player is using any throwable weapon.
	local PWeapon = player(id, "weapontype")
	if ((PWeapon > 50 and PWeapon < 55) or (PWeapon > 71 and PWeapon < 74) or (PWeapon > 74 and PWeapon < 78) or (PWeapon > 85 and PWeapon < 88) or PWeapon == 89) then
		PWeapon = nil
		return true
	end
	PWeapon = nil
	return false
end

function PlayerIsUsingClaw(id) -- Check if the specified player is using a claw.
	local PWeapon = player(id, "weapontype")
	if (PWeapon == 78) then
		PWeapon = nil
		return true
	end
	PWeapon = nil
	return false
end

--function GetXYInFrontOfPlayer(PlayerID, Offset) -- This function is used directly in B2PT_TrailsUpdate() for optimization purposes.
--	local PX, PY, PRot = player(PlayerID, "x"), player(PlayerID, "y"), player(PlayerID, "rot")
--	PRot = rad((PRot - 90)) -- By Flacko.
--	PX = (PX + (Offset * cos(PRot)))
--	PY = (PY + (Offset * sin(PRot)))
--	return PX, PY
--end

-- [HOOKED FUNCTIONS]

addhook("spawn", "B2PT_CreateTrailImagesOnSpawn")
function B2PT_CreateTrailImagesOnSpawn(id) -- Create player trails based on the player's chosen look.
	local PTeam, PSkin = player(id, "team"), player(id, "look")
	if (PTeam == 1) then -- If the player is a Terrorist.
		if (game("sv_gamemode") == "4") then -- If it's Zombies! gamemode, then make the zombies get their respective trails.
			PSkin = "<spritesheet:gfx/npc/zombie.bmp:32:32:m>"
		else
			PSkin = "<spritesheet:gfx/player/t"..(PSkin + 1 )..".bmp:32:32:m>"
		end
	elseif (PTeam == 2) then -- If the player is a Counter-Terrorist.
		PSkin = "<spritesheet:gfx/player/ct"..(PSkin + 1 )..".bmp:32:32:m>"
	elseif (PTeam == 3) then -- If the player is a VIP.
		PSkin = "<spritesheet:gfx/player/vip.bmp:32:32:m>"
	end
	for j = 1, B2PT_MaxTrails do -- For optimization purposes, player trails are created only once (on spawn).
		if (B2PT.Trails[id].Trail[j].PImage ~= nil) then
			freeimage(B2PT.Trails[id].Trail[j].PImage)
			B2PT.Trails[id].Trail[j].PImage = nil
		end
		B2PT.Trails[id].Trail[j].PImage = image(PSkin, 0, 0, 0) -- They just get moved and are made visible / faded when needed.
		imagealpha(B2PT.Trails[id].Trail[j].PImage, 0.0)
	end
	PTeam, PSkin = nil
end

addhook("startround", "B2PT_ResetOnStartRound")
function B2PT_ResetOnStartRound(mode) -- Reset all player trails on round start to avoid bugs.
	for i = 1, 32 do
		if ((player(i, "health") or 0) > 0) then
			B2PT_CreateTrailImagesOnSpawn(i)
		end
	end
end

addhook("startround_prespawn", "B2PT_ResetOnStartRoundPreSpawn")
function B2PT_ResetOnStartRoundPreSpawn(mode)
	for i = 1, 32 do
		for j = 1, B2PT_MaxTrails do
			B2PT.Trails[i].Trail[j].PImage = nil
			B2PT.Trails[i].Trail[j].WImage = nil
		end
	end
end

print("[B2PT] \"Adding\" function 'B2PT_TrailUpdate' to hook \"'ms100'\".")
timer(100, "B2PT_TrailUpdate", "", 0) -- Not an actual hook, but the hook "ms100" has a memory leak at the moment...
function B2PT_TrailUpdate() -- Update the player trails.
	local PTeam, PSkin, PFrame, PRot, WOffsetX, WOffsetY, WRot, PWeapon = 0, 0, 0, 0, 0, 0, 0, 0
	for i = 1, 32 do
		PTeam = player(i, "team") -- Only update the trails when the player's position has changed.
		if (B2PT.Enabled and player(i, "exists") and PTeam > 0 and player(i, "health") > 0 and (B2PT.Trails[i].PX ~= player(i, "x") or B2PT.Trails[i].PY ~= player(i, "y"))) then
			B2PT.Trails[i].PX = player(i, "x")
			B2PT.Trails[i].PY = player(i, "y")
			PRot = player(i, "rot")
			PWeapon = player(i, "weapontype")
			if (player(i, "reloading")) then -- Change trail frames based on the player's equipped weapon/item.
				PFrame = 1
			elseif (PlayerIsUsingPrimaryWeapon(i)) then
				PFrame = 5
			elseif (PlayerIsUsingSecondaryWeapon(i)) then
				PFrame = 4
			elseif (PlayerIsUsingClaw(i)) then
				PFrame = 3
			else
				PFrame = 1
			end
			imageframe(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].PImage, PFrame) -- Update trail frame and position.
			imagepos(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].PImage, B2PT.Trails[i].PX, B2PT.Trails[i].PY, PRot)
			imagealpha(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].PImage, 1.0) -- Make the trail visible.
			tween_alpha(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].PImage, ((B2PT_MaxTrails + 1) * 100), 0.0) -- Make the trail fade out.
			if (PWeapon == 0) then -- Sometimes the game reports the player's equipped weapon as weapon ID 0 after he/she throws a grenade.
				if (B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].CWeapon == 0) then -- In this case set the weapon trail as a knife.
					PWeapon = 50
					B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].CWeapon = 50
				else
					PWeapon = B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].CWeapon
				end
			end
			if (PWeapon ~= B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].CWeapon) then -- Like player trails, weapon trails are also only created when needed,
				if (B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage ~= nil) then -- but are deleted and recreated if the player's equipped weapon changes.
					freeimage(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage)
				end
				B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage = image("gfx/weapons/"..B2PT.WImages[PWeapon].."<m>", 0, 0, 0)
			else
				if (B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage == nil) then
					B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage = image("gfx/weapons/"..B2PT.WImages[PWeapon].."<m>", 0, 0, 0)
				end
			end
			B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].CWeapon = PWeapon
			if (PlayerIsUsingThrowable(i)) then -- It's here where GetXYInFrontOfPlayer() is used directly.
				WRot = rad((PRot - 90)) -- This calculates the weapon trail image position based on the player's rotation.
				WOffsetX = (B2PT.Trails[i].PX + (14 * cos(WRot)))
				WOffsetY = (B2PT.Trails[i].PY + (14 * sin(WRot)))
			else
				WRot = rad((PRot - 90))
				WOffsetX = (B2PT.Trails[i].PX + (17 * cos(WRot)))
				WOffsetY = (B2PT.Trails[i].PY + (17 * sin(WRot)))
			end
			imagepos(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage, WOffsetX, WOffsetY, PRot)
			imagealpha(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage, 1.0)
			tween_alpha(B2PT.Trails[i].Trail[B2PT.Trails[i].CTrail].WImage, ((B2PT_MaxTrails + 1) * 100), 0.0)
			B2PT.Trails[i].CTrail = B2PT.Trails[i].CTrail + 1
			if (B2PT.Trails[i].CTrail == (B2PT_MaxTrails + 1)) then -- Trails are looped through on each player trail update.
				B2PT.Trails[i].CTrail = 1
			end
		end
	end
	PTeam, PSkin, PFrame, PRot, WOffsetX, WOffsetY, WRot, PWeapon = nil, nil, nil, nil, nil, nil, nil, nil
end

addhook("say", "B2PT_CommandCheck", 2) -- Change the priority of this hook if it conflicts with another script.
function B2PT_CommandCheck(id, message)
	if (message == "!pt" and player(id, "rcon") == true) then
		if (B2PT.Enabled == false) then
			B2PT.Enabled = true
			parse("sv_msg 153153255[!] Player Trails activated.")
		else
			B2PT.Enabled = false
			parse("sv_msg 153153255[!] Player Trails deactivated.")		
		end
		return 1
	end
end

print("[B2PT] Successfully loaded!")
